@@ -1,4 +1,5 @@
 from fedbase.utils.data_loader import data_process, log
+from fedbase.utils.tools import add_
 from fedbase.nodes.node import node
 from fedbase.server.server import server_class
 import torch
@@ -7,7 +8,8 @@ import torch.optim as optim
 import os
 from functools import partial
 
-def run(dataset_splited, batch_size, num_nodes, model, objective, optimizer, global_rounds, local_steps, device = torch.device('cuda' if torch.cuda.is_available() else 'cpu'), log_file = True):
+def run(dataset_splited, batch_size, num_nodes, model, objective, optimizer, global_rounds, local_steps,\
+     device = torch.device('cuda' if torch.cuda.is_available() else 'cpu'), log_file=True, finetune=False, finetune_steps = None):
     # dt = data_process(dataset)
     # train_splited, test_splited = dt.split_dataset(num_nodes, split['split_para'], split['split_method'])
     train_splited, test_splited, split_para = dataset_splited
@@ -35,11 +37,11 @@ def run(dataset_splited, batch_size, num_nodes, model, objective, optimizer, glo
 
     # initialize parameters to nodes
     server.distribute([nodes[i].model for i in range(num_nodes)])
-
     weight_list = [nodes[i].data_size/sum([nodes[i].data_size for i in range(num_nodes)]) for i in range(num_nodes)]
+    
     # train!
-    for i in range(global_rounds):
-        print('-------------------Global round %d start-------------------' % (i))
+    for t in range(global_rounds):
+        print('-------------------Global round %d start-------------------' % (t))
         # single-processing!
         for j in range(num_nodes):
             nodes[j].local_update_steps(local_steps, partial(nodes[j].train_single_step))
@@ -51,8 +53,19 @@ def run(dataset_splited, batch_size, num_nodes, model, objective, optimizer, glo
             nodes[j].local_test()
         server.acc(nodes, weight_list)
 
-    # log
-    if log_file:
-        log(os.path.basename(__file__)[:-3] + '_' + split_para, nodes, server)
-
-    return server.model
\ No newline at end of file
+    if not finetune:
+        # log
+        if log_file:
+            log(os.path.basename(__file__)[:-3] + add_(split_para), nodes, server)
+        return server.model
+    else:
+        if not finetune_steps:
+            finetune_steps = local_steps
+        # fine tune
+        for j in range(num_nodes):
+            nodes[j].local_update_steps(finetune_steps, partial(nodes[j].train_single_step))
+            nodes[j].local_test()
+        server.acc(nodes, weight_list)
+        # log
+        log(os.path.basename(__file__)[:-3] + add_('finetune') + add_(split_para), nodes, server)
+        return [nodes[i].model for i in range(num_nodes)]
\ No newline at end of file
