@@ -1,5 +1,6 @@
 from fedbase.utils.data_loader import data_process, log
 from fedbase.utils.visualize import dimension_reduction
+from fedbase.utils.tools import add_
 from fedbase.nodes.node import node
 from fedbase.server.server import server_class
 import torch
@@ -38,7 +39,7 @@ def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer,
     del train_splited, test_splited
 
     # initialize parameters to nodes
-    server.distribute(nodes, list(range(num_nodes)))
+    server.distribute([nodes[i].model for i in range(num_nodes)])
 
     # initialize K cluster model
     cluster_models = [model().to(device) for i in range(K)]
@@ -46,21 +47,23 @@ def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer,
     # train!
     # b_list = []
     # uu_list = []
+    weight_list = [nodes[i].data_size/sum([nodes[i].data_size for i in range(num_nodes)]) for i in range(num_nodes)]
     for i in range(global_rounds):
         print('-------------------Global round %d start-------------------' % (i))
 
         # local update
+        server.model.load_state_dict(server.aggregate([nodes[i].model for i in range(num_nodes)], weight_list))
         for j in range(num_nodes):
             if i == 0:
                 nodes[j].local_update_steps(local_steps, partial(nodes[j].train_single_step))
             elif i < warmup_rounds:
                 nodes[j].local_update_steps(local_steps, partial(nodes[j].train_single_step_con, \
                     model_sim = cluster_models[nodes[j].label], model_all = cluster_models, tmp = tmp, mu = mu, base = None\
-                        , reg_lam = reg_lam, reg_model = server.aggregate(nodes, list(range(num_nodes)))))
+                        , reg_lam = reg_lam, reg_model = server.model))
             else:
                 nodes[j].local_update_steps(local_steps, partial(nodes[j].train_single_step_con, \
                     model_sim = cluster_models[nodes[j].label], model_all = cluster_models, tmp = tmp, mu = mu, base = base\
-                        , reg_lam = reg_lam, reg_model = server.aggregate(nodes, list(range(num_nodes)))))
+                        , reg_lam = reg_lam, reg_model = server.model))
                 
         # # tsne or pca plot
         # # if i == global_rounds-1:
@@ -86,20 +89,20 @@ def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer,
 
         # server aggregation and distribution by cluster
         for j in range(K):
-            server.aggregate(nodes, [i for i in list(range(num_nodes)) if nodes[i].label==j])
-            server.distribute(nodes, [i for i in list(range(num_nodes)) if nodes[i].label==j])
-            cluster_models[j].load_state_dict(server.model.state_dict())
+            assign_ls = [i for i in list(range(num_nodes)) if nodes[i].label==j]
+            weight_ls = [nodes[i].data_size/sum([nodes[i].data_size for i in assign_ls]) for i in assign_ls]
+            model_k = server.aggregate([nodes[i].model for i in assign_ls], weight_ls)
+            server.distribute([nodes[i].model for i in assign_ls], model_k)
+            cluster_models[j].load_state_dict(model_k)
 
         # test accuracy
         for j in range(num_nodes):
             nodes[j].local_test()
-        server.acc(nodes, list(range(num_nodes)))
+        server.acc(nodes, weight_list)
     
+    assign = [[i for i in range(num_nodes) if nodes[i].label == k] for k in range(K)]
     # log
-    if not reg_lam:
-        log(os.path.basename(__file__)[:-3] + '_' + base + '_'+ str(K)  + '_' + split_para, nodes, server)
-    else:
-        log(os.path.basename(__file__)[:-3] + '_' + base + '_'+ str(K) + '_' + str(reg_lam) + '_' + split_para, nodes, server)
+    log(os.path.basename(__file__)[:-3] + add_(K) + add_(base) + add_(tmp) + add_(mu) + add_(reg_lam) + add_(split_para), nodes, server)
 
-    return cluster_models
+    return cluster_models, assign
     
\ No newline at end of file
